/*
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

package complexityruleextraction;

import java.util.ArrayList;

/**
 *
 * @author Julián Luengo Martín
 * @date 2012
 */
public class IntervalValidation {
    
    ArrayList<ArrayList<Limits>> good_intervals;
    ArrayList<ArrayList<Limits>> bad_intervals;
    
    ArrayList<ArrayList<String>> output_good_intervals;
    ArrayList<ArrayList<String>> output_bad_intervals;
    
    ArrayList<dcTrainingTest> datasets;
    
    public IntervalValidation(){
        good_intervals = new ArrayList<ArrayList<Limits>>();
        bad_intervals = new ArrayList<ArrayList<Limits>>();
        //we create 12 arraylist, one per each DCM
        for(int i = 0; i < 12; i++){
            good_intervals.add(new ArrayList<Limits>());
            bad_intervals.add(new ArrayList<Limits>());
        }
    
        datasets = new ArrayList<dcTrainingTest>();
        
        output_good_intervals = new ArrayList<ArrayList<String>>();
        output_bad_intervals = new ArrayList<ArrayList<String>>();
    }
    
    /**
     * Read and parse a text file with a previous output of intervals in order to store  
     * and validate them against a new bunch of data sets given by method 'matchIntervals'
     * @param file The path to the text file with the stored intervals
     */
    public void readIntervals(String file){
        String out[], tab[], line;
        String lines = Fichero.leeFichero(file);
        Limits ival;
        int dc;
        
        //split the table of intervals by lines
        out = lines.split(System.getProperty("line.separator"));
        if(out.length <=1){
            out = lines.split("\\n");
        }
        //discard the first line as it contains the column labels
        for(int i=1;i<out.length;i++){
            line = out[i];
            
            //split by tabs
            tab = line.split("\\t");
            
            //pick the interval if any
            if(tab.length > 1){
                ival = getAsInterval(tab[1]);

                //add the interval to the convenient arraylist
                addInterval(tab[0], ival);
            }
        }
    }
    
    /**
     * Once a set of intervals has been stored using method 'readIntervals'
     * they are matched agains a new bunch of data sets by this method
     * @param file The path to a text file with the new bunch of data sets to be validated
     */
    public void matchIntervals(String file){
        String sp[];
        String data = Fichero.leeFichero(file);
        dcTrainingTest dataset;
        ArrayList<String> covered;
        ArrayList<Limits> intervals;
        Limits ival;
        
        sp = data.split(System.getProperty("line.separator"));
        if(sp.length <=1){
            sp = data.split("\\n");
        }

        //parse the data sets to be validated
        for (int i = 0; i < sp.length; i++) {
            datasets.add(IntervalExtract.processTableLine(sp[i]));
        }
        
        //now go through all the collected data sets and check the membership to each interval
        for (int j = 0; j < 12; j++) {
            //first lets test the good intervals for this DCM
            intervals = good_intervals.get(j);

            //for each good interval of this DCM
            for (int k = 0; k < intervals.size(); k++) {
                ival = intervals.get(k);
                
                covered = new ArrayList<String>();
                covered.add(dcTrainingTest.dcNames[j]+"-good-"+k);
                //search for the covered data sets
                for (int i = 0; i < datasets.size(); i++) {
                    dataset = datasets.get(i);

                    if(ival.contains(dataset.getValues()[j])){
                        covered.add(dataset.getDatasetName());
                    }
                }
                output_good_intervals.add(covered);
            }
            
            //now lets go for the bad intervals of this DCM
            intervals = bad_intervals.get(j);
            for (int k = 0; k < intervals.size(); k++) {
                ival = intervals.get(k);
                
                covered = new ArrayList<String>();
                covered.add(dcTrainingTest.dcNames[j]+"-bad-"+k);
                //search for the covered data sets
                for (int i = 0; i < datasets.size(); i++) {
                    dataset = datasets.get(i);

                    if(ival.contains(dataset.getValues()[j])){
                        covered.add(dataset.getDatasetName());
                    }
                }
                output_bad_intervals.add(covered);
            }
        }
        
        /*
         * once the data sets have been matched, we can print out the item sets with
         * the good and the bad data sets
         */
        
        printIntervalsToFile();
        
    }
    
    /**
     * Checks which Data Complexity Measure is from the string provided
     * looking up its content
     * @param s the string to be identified
     * @return an integer containing the codification of the DCM as given in dcTrainingTest.java
     */
    public int checkDCM(String s){
        if(s.contains("F1")){
            return dcTrainingTest._F1_;
        }else if(s.contains("F2")){
            return dcTrainingTest._F2_;
        }else if(s.contains("F3")){
            return dcTrainingTest._F3_;
        }else if(s.contains("N1")){
            return dcTrainingTest._N1_;
        }else if(s.contains("N2")){
            return dcTrainingTest._N2_;
        }else if(s.contains("N3")){
            return dcTrainingTest._N3_;
        }else if(s.contains("N4")){
            return dcTrainingTest._N4_;
        }else if(s.contains("L1")){
            return dcTrainingTest._L1_;
        }else if(s.contains("L2")){
            return dcTrainingTest._L2_;
        }else if(s.contains("L3")){
            return dcTrainingTest._L3_;
        }else if(s.contains("T1")){
            return dcTrainingTest._T1_;
        }else if(s.contains("T2")){
            return dcTrainingTest._T2_;
        }
        
        //should never happen
        return -1;
    }
    
    /**
     * Parses an string in the form "[<lowbound>,<upperbound>]" to a 
     * pair of doubles
     * @param s the string with the intervals in a text readable form
     * @return an array with the limits, being index 0 the lower one and index 1 the higher one
     */
    protected Limits getAsInterval(String s){
        String out[];
        Limits bounds = new Limits();
        
        out = s.split(",");
       
        bounds.down = Double.valueOf(out[0].substring(1));
        bounds.up = Double.valueOf(out[1].substring(0,out[1].lastIndexOf("]")));
        
        return bounds; 
    }
    
    /**
     * Add an interval to its corresponding interval set, from the name.
     * The name can have any content, but the name of the DCM (F1, F2, F3, etc.) and
     * whether is good or bad ("good" and "bad" respectively) must be present in the
     * string
     * @param ivalName the name of the interval
     * @param ival the bounds of the interval
     */
    protected void addInterval(String ivalName, Limits ival){

        //pick the complexity measure
        int dc = checkDCM(ivalName);
        
        //check whether is good or bad interval
        if (ivalName.contains("good")) {
            good_intervals.get(dc).add(ival);
        } else {
            bad_intervals.get(dc).add(ival);
        }
     
    }
    
    /**
     * Prints the data sets into a text file, classifying them as good or bad behavior intervals
     * according to the 'matchIntervals' method.
     */
    protected void printIntervalsToFile(){
        int maxSize;
        String data;
        
        //Print the good rules' itemsets to file
        maxSize = -1;
        for (int i = 0; i < output_good_intervals.size(); i++) {
            maxSize = Math.max(output_good_intervals.get(i).size(), maxSize);
        }
        data = "";
        for (int j = 0; j < maxSize; j++) {
            for (int i = 0; i < output_good_intervals.size(); i++) {
                if (output_good_intervals.get(i).size() > j) {
                    data += output_good_intervals.get(i).get(j);
                }
                data += "\t";
            }
            data += System.getProperty("line.separator");
        }
        Fichero.escribeFichero("good.txt", data);
        
        //Print the bad rules' itemsets to file
        maxSize = -1;
        for (int i = 0; i < output_bad_intervals.size(); i++) {
            maxSize = Math.max(output_bad_intervals.get(i).size(), maxSize);
        }
        data = "";
        for (int j = 0; j < maxSize; j++) {
            for (int i = 0; i < output_bad_intervals.size(); i++) {
                if (output_bad_intervals.get(i).size() > j) {
                    data += output_bad_intervals.get(i).get(j);
                }
                data += "\t";
            }
            data += System.getProperty("line.separator");
        }
        Fichero.escribeFichero("bad.txt", data);
    }
}
