/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

/*
 *    IntervalExtract.java
 *    Copyright (C) 2010 Julian Luengo
 *
 */
package complexityruleextraction;

/**
 *
 * @author Julián Luengo Martín
 */
public class dcTrainingTest implements Comparable {

    public static int compareby = 0;
    public final  static int _F1_ = 0;
    public final  static int _F2_ = 1;
    public final  static int _F3_ = 2;
    public final  static int _N1_ = 3;
    public final  static int _N2_ = 4;
    public final  static int _N3_ = 5;
    public final  static int _N4_ = 6;
    public final  static int _L1_ = 7;
    public final  static int _L2_ = 8;
    public final  static int _L3_ = 9;
    public final  static int _T1_ = 10;
    public final  static int _T2_ = 11;

    public double F1;
    public double F2;
    public double F3;
    public double N1;
    public double N2;
    public double N3;
    public double N4;
    public double L1;
    public double L2;
    public double L3;
    public double T1;
    public double T2;
    //Unused variables... yet
    public double NPos; /** number of positive instances for imbalanced problems*/
    public double NNeg; /** number of negative instances for imbalanced problems*/
    public double IR;   /** Imbalanced ratio defined as NPos/NNeg */

    public static String dcNames[] = {"F1","F2","F3","N1","N2","N3","N4","L1","L2","L3","T1","T2","NPos","NNeg","IR"};

    public String datasetName = "";
    public double trainingAcc = 0;
    public double testAcc = 0;

    /**
     * Set the accuracy values obtained for this data set
     * @param tra training accuracy
     * @param tst test accuracy
     */
    public void setAccuracy(double tra,double tst){
        trainingAcc = tra;
        testAcc = tst;
    }

    /**
     * Sets the data complexity measure considered for sorting
     * @param _whichDC the data complexity measure (_F1_, _F2_, etc.).
     */
    public static void compareBy(int _whichDC){
        compareby = _whichDC;
    }

    /**
     * Sets the name of the data set associated
     * @param name the name of the data set
     */
    public void setName(String name){
        datasetName = new String(name);
    }

    /**
     * Sets the F1 value of the data set
     * @param f1 the f1 data complexity measure
     */
    public void setF1(double f1){
        F1 = f1;
    }

    /**
     * Sets the F2 value of the data set
     * @param f2 the f1 data complexity measure
     */
    public void setF2(double f2){
        F2 = f2;
    }

    /**
     * Sets the F3 value of the data set
     * @param f3 the f1 data complexity measure
     */
    public void setF3(double f3){
        F3 = f3;
    }

    /**
     * Sets the N1 value of the data set
     * @param n1 the f1 data complexity measure
     */
    public void setN1(double n1){
        N1 = n1;
    }

    /**
     * Sets the N2 value of the data set
     * @param n2 the f1 data complexity measure
     */
    public void setN2(double n2){
        N2 = n2;
    }

    /**
     * Sets the N3 value of the data set
     * @param n3 the f1 data complexity measure
     */
    public void setN3(double n3){
        N3 = n3;
    }

    /**
     * Sets the N4 value of the data set
     * @param n4 the f1 data complexity measure
     */
    public void setN4(double n4){
        N4 = n4;
    }

    /**
     * Sets the L1 value of the data set
     * @param l1 the f1 data complexity measure
     */
    public void setL1(double l1){
        L1 = l1;
    }

    /**
     * Sets the L2 value of the data set
     * @param l2 the f1 data complexity measure
     */
    public void setL2(double l2){
        L2 = l2;
    }

    /**
     * Sets the L3 value of the data set
     * @param l3 the f1 data complexity measure
     */
    public void setL3(double l3){
        L3 = l3;
    }

    /**
     * Sets the T1 value of the data set
     * @param t1 the f1 data complexity measure
     */
    public void setT1(double t1){
        T1 = t1;
    }

    /**
     * Sets the T2 value of the data set
     * @param t2 the f1 data complexity measure
     */
    public void setT2(double t2){
        T2 = t2;
    }
    
    /**
     * Sets the NPos value of the data set
     * @param npos the NPos data complexity measure
     */
    public void setNPos(double npos){
        NPos = npos;
    }
    
    /**
     * Sets the NNeg value of the data set
     * @param nneg the NNeg data complexity measure
     */
    public void setNNeg(double nneg){
        NNeg = nneg;
    }
    
    /**
     * Sets the IR value of the data set
     * @param ir the IR data complexity measure
     */
    public void setIR(double ir){
        IR = ir;
    }

    public String toString() {
        String str = new String(this.datasetName+"\t");

        str += String.valueOf(F1) + "\t";
        str += String.valueOf(F2) + "\t";
        str += String.valueOf(F3) + "\t";
        str += String.valueOf(N1) + "\t";
        str += String.valueOf(N2) + "\t";
        str += String.valueOf(N3) + "\t";
        str += String.valueOf(N4) + "\t";
        str += String.valueOf(L1) + "\t";
        str += String.valueOf(L2) + "\t";
        str += String.valueOf(L3) + "\t";
        str += String.valueOf(T1) + "\t";
        str += String.valueOf(T2) + "\t";
        str += String.valueOf(NPos) + "\t";
        str += String.valueOf(NNeg) + "\t";
        str += String.valueOf(IR);

        return str;
    }

    public static String header() {
        String str = new String();

        str += "F1\t";
        str += "F2\t";
        str += "F3\t";
        str += "N1\t";
        str += "N2\t";
        str += "N3\t";
        str += "N4\t";
        str += "L1\t";
        str += "L2\t";
        str += "L3\t";
        str += "T1\t";
        str += "T2\t";
        str += "NPos\t";
        str += "NNeg\t";
        str += "IR";

        return str;

    }

    /**
     * Gets the data complexity values associated to this data set
     * @return an array with the data complexity measures
     */
    public double[] getValues() {
        double values[] = new double[15];

        values[0] = F1;
        values[1] = F2;
        values[2] = F3;
        values[3] = N1;
        values[4] = N2;
        values[5] = N3;
        values[6] = N4;
        values[7] = L1;
        values[8] = L2;
        values[9] = L3;
        values[10] = T1;
        values[11] = T2;
        values[12] = NPos;
        values[13] = NNeg;
        values[14] = IR;

        return values;
    }

    /**
     * Gets the data set unique name
     * @return the data set's name
     */
    public String getDatasetName(){
        return datasetName;
    }

    /**
     * Gets the training accuracy obtained for this data set
     * @return the training accuracy
     */
    public double getTrainingAccuracy(){
        return trainingAcc;
    }

    /**
     * Gets the test accuracy obtained for this data set
     * @return the test accuracy
     */
    public double getTestAccuracy(){
        return testAcc;
    }

    public int compareTo(Object o){
        dcTrainingTest dc = (dcTrainingTest) o;
        double dc1,dc2;

        switch(compareby){
            case 0: dc1 = this.F1; dc2 = dc.F1;break;
            case 1: dc1 = this.F2; dc2 = dc.F2;break;
            case 2: dc1 = this.F3; dc2 = dc.F3;break;
            case 3: dc1 = this.N1; dc2 = dc.N1;break;
            case 4: dc1 = this.N2; dc2 = dc.N2;break;
            case 5: dc1 = this.N3; dc2 = dc.N3;break;
            case 6: dc1 = this.N4; dc2 = dc.N4;break;
            case 7: dc1 = this.L1; dc2 = dc.L1;break;
            case 8: dc1 = this.L2; dc2 = dc.L2;break;
            case 9: dc1 = this.L3; dc2 = dc.L3;break;
            case 10: dc1 = this.T1; dc2 = dc.T1;break;
            case 11: dc1 = this.T2; dc2 = dc.T2;break;
            case 12: dc1 = this.NPos; dc2 = dc.NPos;break;
            case 13: dc1 = this.NNeg; dc2 = dc.NNeg;break;
            case 14: dc1 = this.IR; dc2 = dc.IR;break;
            default: dc1 = dc2 = 0;
        }

        if(dc1 < dc2)
            return -1;
        if(dc1 > dc2)
            return 1;
        return 0;
    }
}
